export default function Barcode_checksum(barcode: string): number {
    return (new EAN(barcode)).correct; // '0', '-1' or '-2'
}

enum Result {
    Correct = 0,
    Incorrect = -1,
    Invalid_format = -2
}

class EAN {
    static readonly Format: RegExp = new RegExp("^(?!000)(\\d{13}$)");
    static readonly Thirteen: number = 13;
    private readonly _data: Array<number> = new Array;
    private _correct: number = Result.Invalid_format; // Default value...
    get correct(): number {
        return this._correct;
    }

    constructor(barcode: string) {
        if (barcode.match(EAN.Format)) {
            barcode.split("").every((s: string) => { // 'barcode' is divided into individual numbers...
                const element: number = parseInt(s); // Conversion for computations...
                if (isNaN(element)) return false; // 'every' stops with 'this._correct === Result.Invalid_format'...
                this._data.push(element);
                return true; // 'every' goes on...
            });
            this._correct = this._checksum() === this._data[EAN.Thirteen - 1] ? Result.Correct : Result.Incorrect;
        }
    }

    private _checksum(): number {
        // Méthode : https://fr.wikipedia.org/wiki/EAN_13#Calcul_de_la_cl%C3%A9_de_contr%C3%B4le_EAN_13
        // '471-9-5120-0288-x' with 'x' as checksum (i.e., 'x === 9')
        // '7', '9', '1', '0', '2', '8'
        const remainder = (this._data.filter((element: number, index: number) => index % 2 !== 0)
                .reduce((result, element) => result + 3 * element, 0) +
            // '4', '1', '5', '2', '0', '8'
            this._data.filter((element: number, index: number) => index % 2 === 0 && index !== EAN.Thirteen - 1)
                .reduce((result, element) => result + element, 0)) % 10;
        return remainder === 0 ? 0 : 10 - remainder;
    }
}
